#!/bin/ksh

# Licensed Materials - Property of IBM
# XL Compiler Usage Logger Tool V1.0
# Copyright IBM Corp. 2011. All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

# This script is a wrapper designed to gather information about who is
# executing the XL compilers on a machine.
# 
# The script works by replacing the invocation commands in vac/bin and
# vacpp/bin, thus getting executed by the user when they run the com;piler.  It
# collects data and creates a file in a directory specified by
# $XL_COMP_USER_DIR which contains that data.

#########
# OPTIONS
#########

# Set TRACKMORE to 1 in order to track more of the invocations by users
# Records the time that an invocation took place (down to the second).
# CAVEAT 1:  Please note that this will not record ALL invocations -
# invocations # within the same second will be missed
# CAVEAT 2:  Depending on compiler invocation volume, a large number of files
# could be created by this script under the directory specified by
# $XL_COMP_USER_DIR and could use significant index space.
TRACKMORE=0

# Set DISABLELOGGING to 1 if you wish to disable logging
DISABLELOGGING=0

# Can we successfully log?  Assume yes until we prove otherwise
LOGACCESS=1

# The name of the directory where we will create files.  By default we will try
# to use /var/tmp/xl_comp_users
XL_COMP_USER_DIR=${XL_COMP_USER_DIR:-/var/tmp/xl_comp_users}

# Set HUSH to 0 if you want error messages from this script to be emitted
HUSH=1

#############
# END OPTIONS
#############

filterOutput() {
   if [ $HUSH == 1 ]; then
      "$@" >/dev/null 2>&1
   else
      "$@"
   fi
}

# Create the log directory if it does not exist and logging isn't disabled
if [ ! -e $XL_COMP_USER_DIR ] && [ $DISABLELOGGING -eq 0 ]; then
   OLDUMASK=`umask`

   # Set umask to ensure rwxrwxrwx if we're creating the log
   filterOutput umask 000 

   # Create the log directory
   filterOutput mkdir -p $XL_COMP_USER_DIR

   # Reset umask to its original state
   filterOutput umask $OLDUMASK 
fi

# Does our logging directory exist, and is it writable?
if [ ! -w $XL_COMP_USER_DIR ] || [ ! -d $XL_COMP_USER_DIR ]; then
   LOGACCESS=0
fi


# If we can log, log the data.
if [ $LOGACCESS -eq 1 ]  && [ $DISABLELOGGING -eq 0 ]; then

   # Collect information about who is executing this
   UID=`id -u`
   USERNAME=`id -u -n` 
   HOSTNAME=`hostname`
   if (( $TRACKMORE == 1 )) then
      DATE=`date +%s`
   else
      DATE=""
   fi

   # Set umask to ensure rw-rw-rw- if we're creating the log
   OLDUMASK=`umask`
   filterOutput umask 111 

   # Create a file to indicate usage by this user id
   if [ $TRACKMORE -eq 1 ]; then
      filterOutput touch "$XL_COMP_USER_DIR/${USERNAME}_${UID}_${HOSTNAME}_${DATE}" 
   else
      filterOutput touch "$XL_COMP_USER_DIR/${USERNAME}_${UID}_${HOSTNAME}"  
   fi

   # Reset umask to its original state
   filterOutput umask $OLDUMASK 

fi


# Determine compiler location and command called
D=`dirname $0`
B=`basename $0`

# Execute the actual compiler
exec $D/.orig/$B "$@"

